// kinc_companion
/*
	Helper functions for companions.
*/
// ChazM 1/3/07
// ChazM 1/9/07 - added str refs
// ChazM 1/30/07 - Feat gain support
// ChazM 1/30/07 - inventory access, possession, leave party, and removal of feats support
// ChazM 2/2/07 - "OneOfMany" updated, influence changes now reported.
// ChazM 2/12/07 - added GetInfluenceBandStrRef(), DoInfluenceEffects() now updates vars for use by GUI
// ChazM 2/22/07 - bug fixes
// ChazM 5/3/07 - Companion Initialization
// ChazM 5/4/07 - moved hangout functions to ginc_companion
// ChazM 5/30/07 - added GetHigherInfluence(), GetHighestCompanionInfluenceInParty
// ChazM 6/4/07 - fixed STR_REF_ONEOFMANY
// ChazM 6/5/07 - updated ReportInfluenceChange()
// ChazM 6/6/07 - moved reporting of influence from SetInfluenceByCode() to ModifyInfluenceByCode() since what we want to report is a delta.
// ChazM 6/14/07 - added constants
// ChazM 7/3/07 - companion feat experimental changes
// ChazM 7/5/07 - more updates to ReportInfluenceChange()
// MDiekmann 7/5/07 - added variable for Safiya's disguise
// ChazM 7/10/07 - Updated InitiateLeaveParty() - conversation name fix.
// ChazM 7/17/07 - added influence gain/loss sounds
// ChazM 7/19/07 - Fix for influence total reported not always accurate when close to min or max
// ChazM 7/25/07 - Safiya never goes below disillusioned (was never below wavering)
// ChazM 8/7/07 - FEAT_IMPROVED_EMPOWER_SPELL tags along with FEAT_INFLUENCE_SAFIYA_DEVOTED
// MDiekmann 9/12/07 - FEAT_IMPROVED_EMPOWER_SPELL tags along with FEAT_INFLUENCE_SAFIYA_LOYAL

// ARH 12/19/09 - Created GetInfluenceAdjustment() for setting initial influence when first meeting companion, added magnitude cap

// void main(){}

#include "ginc_henchman"
#include "ginc_companion"
#include "ginc_object"
#include "ginc_ipspeaker"

//-------------------------------------------------
// Constants
//-------------------------------------------------


// Companion ResRefs
const string RR_SAFIYA 		= "co_safiya";
const string RR_GANN		= "co_gann";
const string RR_OKKU		= "co_okku";
const string RR_DOVE		= "co_dove";
const string RR_ONEOFMANY 	= "co_oneofmany";


// Companion String Refs
const int STR_REF_SAFIYA 	= 185138;
const int STR_REF_GANN		= 185442;
const int STR_REF_OKKU		= 184853;
const int STR_REF_DOVE		= 185506;
const int STR_REF_ONEOFMANY = 185443;

const int	STRING_REF_LOST_INFLUENCE 	= 178842; // Influence lost with
const int	STRING_REF_GAINED_INFLUENCE = 178841; // Influence gained with
const int 	STRING_REF_TOTAL_INFLUENCE	= 208093; // "Total Influence"

// we'll always use tag as the roster name 
const string TAG_SAFIYA 	= "safiya";
const string TAG_GANN		= "gann";
const string TAG_OKKU		= "okku";
const string TAG_DOVE		= "dove";
const string TAG_ONEOFMANY 	= "oneofmany";


// codes used as params in action and conditon scripts
// only the first letter of names are used to promote 
// fault tolerance
const string CODE_SAFIYA 	= "sa";
const string CODE_GANN		= "ga";
const string CODE_OKKU		= "ok";
const string CODE_DOVE		= "do";
const string CODE_ONEOFMANY = "on";

// Influence Levels
const int INFLUENCE_REBELLIOUS 		= -3;
const int INFLUENCE_DISILLUSIONED 	= -2;
const int INFLUENCE_WAVERING 		= -1;
const int INFLUENCE_NEUTRAL 		= 0;
const int INFLUENCE_SUPPORTIVE 		= 1;
const int INFLUENCE_LOYAL 			= 2;
const int INFLUENCE_DEVOTED 		= 3;

// strig Ref's
const int STR_REF_INFLUENCE_REBELLIOUS 		= 185049;
const int STR_REF_INFLUENCE_DISILLUSIONED 	= 185050;
const int STR_REF_INFLUENCE_WAVERING 		= 185051;
const int STR_REF_INFLUENCE_NEUTRAL 		= 185052;
const int STR_REF_INFLUENCE_SUPPORTIVE 		= 185053;
const int STR_REF_INFLUENCE_LOYAL 			= 185054;
const int STR_REF_INFLUENCE_DEVOTED 		= 185055;

// Influence ranges
const int INFLUENCE_RANGE_REBELLIOUS 	= -76;
const int INFLUENCE_RANGE_DISILLUSIONED = -51;
const int INFLUENCE_RANGE_WAVERING 		= -26;
const int INFLUENCE_RANGE_NEUTRAL 		= 25;
const int INFLUENCE_RANGE_SUPPORTIVE 	= 50;
const int INFLUENCE_RANGE_LOYAL 		= 75;
const int INFLUENCE_RANGE_DEVOTED 		= 100;

// Magnitude amounts
const int INFLUENCE_MAGNITUDE_1 		= 6;	
const int INFLUENCE_MAGNITUDE_2 		= 11;
const int INFLUENCE_MAGNITUDE_3 		= 15;
const int INFLUENCE_MAGNITUDE_4 		= 25;
const int INFLUENCE_MAGNITUDE_5 		= 35;

//const int MAX_COMPANIONS	= 5;

// the influence feats
const int FEAT_INFLUENCE_SAFIYA_SUPPORT		= 1895;
const int FEAT_INFLUENCE_SAFIYA_LOYAL		= 1896;
const int FEAT_INFLUENCE_SAFIYA_DEVOTED 	= 1897;
const int FEAT_INFLUENCE_SAFIYA_ROMANCE 	= 1898;
const int FEAT_INFLUENCE_GANN_SUPPORT 		= 1899;
const int FEAT_INFLUENCE_GANN_LOYAL 		= 1900;
const int FEAT_INFLUENCE_GANN_DEVOTED 		= 1901;
const int FEAT_INFLUENCE_GANN_ROMANCE 		= 1902;
const int FEAT_INFLUENCE_OKKU_SUPPORT 		= 1903;
const int FEAT_INFLUENCE_OKKU_LOYAL 		= 1904;
const int FEAT_INFLUENCE_OKKU_DEVOTED 		= 1905;
const int FEAT_INFLUENCE_ONEOFMANY_SUPPORT 	= 1906;
const int FEAT_INFLUENCE_ONEOFMANY_LOYAL 	= 1907;
const int FEAT_INFLUENCE_ONEOFMANY_DEVOTED 	= 1908;
const int FEAT_INFLUENCE_DOVE_SUPPORT 		= 1909;
const int FEAT_INFLUENCE_DOVE_LOYAL 		= 1910;
const int FEAT_INFLUENCE_DOVE_DEVOTED 		= 1911;

// other misc. companion related vars
//const string TAG_SAFIYA_ROBE 		= "safiya_robe";		// tag of Safiya's robe (if this changes, item script names must also change)
const string TAG_SAFIYA_ROBE 		= "nx1_safiyas_robes";		// tag of Safiya's robe
const string VAR_RED_WIZ_RESTRICTED = "RED_WIZ_RESTRICTED";	// area variable: 1 = safiya robes can't be worn here.
const string VAR_RED_WIZ_RESTRICTED_OVERRIDE = "RED_WIZ_RESTRICT_OVRIDE";	// variable on safiya: 1 = safiya can put on robes even if restricted by area
const string VAR_REMOVED_ROBE 		= "TookRobesOff";		// Set to 1 when robes removed, reset when robes reworn
const string TAG_SAFIYA_DISGUISE	= "nx1_safiyas_disguise";

const string SOUND_INFLUENCE_GAIN = "gui_infuencegain01";
const string SOUND_INFLUENCE_LOSS = "gui_infuenceloss01";

//-------------------------------------------------
// Prototypes
//-------------------------------------------------

// Companion add/remove
// --------------------------------------------------------------
void AddCompanionsToRoster();
void SpawnCompanion(string sTag, string sResRef);
void SpawnCompanions();
int RemoveAllCompanions( object oPC , int bDespawn = TRUE);

// Influence
// --------------------------------------------------------------

// -public
void ReportInfluenceChange(string sCompanionCode, int nDelta, int nInfluenceTotal);
int GetInfluenceByCode( string sCompanionCode );
void ModifyInfluenceByCode( string sCompanionCode, int nDelta );


// -private
string GetFirstChars(string sString);
string GetCompanionTagByCode(string sCompanionCode);

string GetInfluenceVarNameByTag( string sCompanionTag );
int GetInfluenceByTag( string sCompanionTag );

struct rNameValuePair GetHigherInfluence(object oPC, struct rNameValuePair stInfluence, string sRosterName);
string GetHighestCompanionInfluenceInParty(object oPC);


void DoRemoveFeat( object oTarget, int iFeat);
void RemoveInfluenceFeats(object oTarget = OBJECT_SELF);
int GetSupportiveFeat(string sTag);
int GetLoyalFeat(string sTag);
int GetDevotedFeat(string sTag);
int GetRomanceFeat(string sTag);

void DoInfluenceEffects(object oTarget = OBJECT_SELF);

void SetInfluenceByCode( string sCompanionCode, int nAmount );
int MagnitudeConversion(int iMagnitude);
int GetInfluenceBandByCode(string sCompanionCode);
int GetInfluenceBand(int iInfluence);
int GetInfluenceBandStrRef(int iInfluenceBand);


//-------------------------------------------------
// Function Definitions
//-------------------------------------------------

// Companion add/remove
// --------------------------------------------------------------


// adds all companions to the roster
// members in roster must still be added to party	
void InitializeNX1Companions()
{
	InitializeCompanion(TAG_SAFIYA, TAG_SAFIYA, RR_SAFIYA);
	InitializeCompanion(TAG_GANN, TAG_GANN, RR_GANN);
	InitializeCompanion(TAG_OKKU, TAG_OKKU, RR_OKKU);
	InitializeCompanion(TAG_DOVE, TAG_DOVE, RR_DOVE);
	InitializeCompanion(TAG_ONEOFMANY, TAG_ONEOFMANY, RR_ONEOFMANY);
}

// spawn all NX1 companions at their hangout
void SpawnNX1CompanionRosterMembersAtHangout()
{
	// tags also serve as the roster name.
	SpawnNonPartyRosterMemberAtHangout(TAG_SAFIYA);
	SpawnNonPartyRosterMemberAtHangout(TAG_GANN);
	SpawnNonPartyRosterMemberAtHangout(TAG_OKKU);
	SpawnNonPartyRosterMemberAtHangout(TAG_DOVE);
	SpawnNonPartyRosterMemberAtHangout(TAG_ONEOFMANY);
}

// Success should return 0, or remaining number of companions on failure
int RemoveAllCompanions( object oPC , int bDespawn = TRUE)
{
	RemoveRosterMemberFromParty( TAG_SAFIYA, 	oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_GANN,	oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_OKKU, 		oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_DOVE, 	oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_ONEOFMANY, 	oPC, bDespawn );
	
	//return GetNumHenchmen( oPC );
	return (0);
}

// Influence
// --------------------------------------------------------------

// placeholder - needs to be updated!
int GetCompanionNameStringRef(string sCompanionCode)
{
	// original companions str refs are 178843 - 178855
	//return ( 178842 + 1 );
	
	int iCompanionStrRef = 0;
	// ensure we have just a single letter companion code
	sCompanionCode = GetFirstChars(sCompanionCode);

	if (sCompanionCode == CODE_SAFIYA)
		iCompanionStrRef = STR_REF_SAFIYA;
	else if (sCompanionCode == CODE_GANN)
		iCompanionStrRef = STR_REF_GANN;
	else if (sCompanionCode == CODE_OKKU)
		iCompanionStrRef = STR_REF_OKKU;
	else if (sCompanionCode == CODE_DOVE)
		iCompanionStrRef = STR_REF_DOVE;
	else if (sCompanionCode == CODE_ONEOFMANY)
		iCompanionStrRef = STR_REF_ONEOFMANY;
	else
		PrettyError("GetCompanionNameStringRef: unrecognized code"); 
		
	return iCompanionStrRef;
}


void ReportInfluenceChange(string sCompanionCode, int nDelta, int nInfluenceTotal)
{
//	int nOldInfluence = GetInfluenceByNumber( nCompanion );
//	IncInfluenceByNumber( nCompanion, nChange );
//	int nNewInfluence = GetInfluenceByNumber( nCompanion );
	
	string sOut;
//	int nDelta = nNewInfluence - nOldInfluence; //nChange; // Actual nDelta may not == nChange
	if (nDelta == 0)
		return; // don't report no change in influence.
	string sSign = "";
	string sSound = "";
	
	if ( nDelta > 0 )
	{
		sOut = GetStringByStrRef( STRING_REF_GAINED_INFLUENCE );
		sSign = "+";
		sSound = SOUND_INFLUENCE_GAIN;
	}		
	else
	{
		sOut = GetStringByStrRef( STRING_REF_LOST_INFLUENCE );
		sSign = "-";
		sSound = SOUND_INFLUENCE_LOSS;
	}
	
	//sOut += " " + GetStringByStrRef(GetCompanionNameStringRef(sCompanionCode)) + ": " + IntToString(abs(nDelta));
	sOut += " " + GetStringByStrRef(GetCompanionNameStringRef(sCompanionCode)) + ": " + sSign + IntToString(abs(nDelta));
	sOut += " (" + GetStringByStrRef( STRING_REF_TOTAL_INFLUENCE ) + ": " + IntToString(nInfluenceTotal) + ")";
	
    object oPC = ( GetPCSpeaker()==OBJECT_INVALID?OBJECT_SELF:GetPCSpeaker() );
	SendMessageToPC( oPC, sOut );
	SetNoticeText( oPC, sOut );
	AssignCommand(oPC, PlaySound(sSound));

	
	// PrettyDebug( sOut );
	//PrettyDebug("Total influence with " + GetStringByStrRef(GetCompanionNameStringRef(nCompanion)) + " = " + IntToString(nNewInfluence));
	//PrettyDebug("Total influence with " + sCompanionCode + " = " + IntToString(nNewInfluence));
}

// The first few chars of the tag defines the code.
string GetFirstChars(string sString)
{
	sString = GetStringLeft(sString, 2);
	sString = GetStringLowerCase (sString);
	return (sString);
}

// sCompanionCode = identifying code letter.  (It only matters that the first letters are correct, so the tag could be passed in as well)
string GetCompanionTagByCode(string sCompanionCode)
{
	string sCompanionTag = "";
	string sCompanionCodeInput = sCompanionCode;

	// ensure we have just a single letter companion code
	sCompanionCode = GetFirstChars(sCompanionCode);

	if (sCompanionCode == CODE_SAFIYA)
		sCompanionTag = TAG_SAFIYA;
	else if (sCompanionCode == CODE_GANN)
		sCompanionTag = TAG_GANN;
	else if (sCompanionCode == CODE_OKKU)
		sCompanionTag = TAG_OKKU;
	else if (sCompanionCode == CODE_DOVE)
		sCompanionTag = TAG_DOVE;
	else if (sCompanionCode == CODE_ONEOFMANY)
		sCompanionTag = TAG_ONEOFMANY;
	else
		PrintString("ginc_companions-GetCompanionTagByCode: Invalid companion Code: " + sCompanionCodeInput); 
		
	return sCompanionTag;
}
/*
// look through all the numbers for this Tag.
int GetCompanionCodeByTag(string sTag)
{
	string sCompareTag;
	int i;
	for (i=1; i<=MAX_COMPANIONS; i++)
	{
		sCompareTag = GetCompanionTagByNumber(i);
		if (sTag == sCompareTag)
			return(i);
	}
	return (0);
}
*/

// Used in ka/kc_influence* scripts
string GetInfluenceVarNameByTag( string sCompanionTag )
{
	string sVarName		= INFLUENCE_PREFIX + sCompanionTag; //INFLUENCE_PREFIX from ginc_companions
	return sVarName;
}

// Return PC's influence of companion by CompanionCode
int GetInfluenceByTag( string sCompanionTag )
{
	string sVar = GetInfluenceVarNameByTag( sCompanionTag );
	int iRet = GetGlobalInt( sVar );
	return (iRet );	
}

// return a name value pair with a tag/roster-name and influence value
// this will be the highest of the 2 items passed in.
struct rNameValuePair GetHigherInfluence(object oPC, struct rNameValuePair stInfluence, string sRosterName)
{
	if (GetIsRosterNameInParty(oPC, sRosterName))
	{
		int iThisInfluence = GetInfluenceByTag(sRosterName); // ok because our roster companions RosterName same as Tag
		if (iThisInfluence > stInfluence.iValue)
		{
			stInfluence.sName = sRosterName;
			stInfluence.iValue = iThisInfluence;
		}
	}
	return (stInfluence);
}

// returns whichever companion the PC has the highest influence with that is currently in the party.
string GetHighestCompanionInfluenceInParty(object oPC)  
{
	struct rNameValuePair stHighestInfluence;
	stHighestInfluence.sName = "";
	stHighestInfluence.iValue = -101; // less than lowest possible influence.	
	
	stHighestInfluence = GetHigherInfluence(oPC, stHighestInfluence, TAG_SAFIYA);
	stHighestInfluence = GetHigherInfluence(oPC, stHighestInfluence, TAG_GANN);
	stHighestInfluence = GetHigherInfluence(oPC, stHighestInfluence, TAG_OKKU);
	stHighestInfluence = GetHigherInfluence(oPC, stHighestInfluence, TAG_DOVE);
	stHighestInfluence = GetHigherInfluence(oPC, stHighestInfluence, TAG_ONEOFMANY);
	
	return (stHighestInfluence.sName);
}

// Return PC's influence of companion by CompanionCode
int GetInfluenceByCode( string sCompanionCode )
{
	string sCompanionTag 	= GetCompanionTagByCode(sCompanionCode);
	return (GetInfluenceByTag(sCompanionTag));	
}

// remove the feat if target has it.
void DoRemoveFeat( object oTarget, int iFeat)
{
	if (GetHasFeat(iFeat, oTarget))
	{
		FeatRemove(oTarget, iFeat);
	}
}

void RemoveInfluenceFeats(object oTarget = OBJECT_SELF)
{
	string sTargetTag = GetTag(oTarget);
	int iFeat;

	iFeat = GetSupportiveFeat(sTargetTag);
	DoRemoveFeat(oTarget, iFeat);
	
	iFeat = GetLoyalFeat(sTargetTag);
	DoRemoveFeat(oTarget, iFeat);

	iFeat = GetDevotedFeat(sTargetTag);
	DoRemoveFeat(oTarget, iFeat);
	if (iFeat == FEAT_INFLUENCE_SAFIYA_DEVOTED || iFeat == FEAT_INFLUENCE_SAFIYA_LOYAL)
	{
		DoRemoveFeat(oTarget, FEAT_IMPROVED_EMPOWER_SPELL);
	}
}


int GetSupportiveFeat(string sTag)
{
	int iFeat = -1;
	if (sTag == TAG_SAFIYA)
		iFeat = FEAT_INFLUENCE_SAFIYA_SUPPORT;
	else if (sTag == TAG_GANN)
		iFeat = FEAT_INFLUENCE_GANN_SUPPORT;
	else if (sTag == TAG_OKKU)
		iFeat = FEAT_INFLUENCE_OKKU_SUPPORT;
	else if (sTag == TAG_DOVE)
		iFeat = FEAT_INFLUENCE_DOVE_SUPPORT;
	else if (sTag == TAG_ONEOFMANY)
		iFeat = FEAT_INFLUENCE_ONEOFMANY_SUPPORT;
	return (iFeat);		
}

int GetLoyalFeat(string sTag)
{
	int iFeat = -1;
	if (sTag == TAG_SAFIYA)
		iFeat = FEAT_INFLUENCE_SAFIYA_LOYAL;
	else if (sTag == TAG_GANN)
		iFeat = FEAT_INFLUENCE_GANN_LOYAL;
	else if (sTag == TAG_OKKU)
		iFeat = FEAT_INFLUENCE_OKKU_LOYAL;
	else if (sTag == TAG_DOVE)
		iFeat = FEAT_INFLUENCE_DOVE_LOYAL;
	else if (sTag == TAG_ONEOFMANY)
		iFeat = FEAT_INFLUENCE_ONEOFMANY_LOYAL;
	return (iFeat);		
}


int GetDevotedFeat(string sTag)
{
	int iFeat = -1;
	if (sTag == TAG_SAFIYA)
		iFeat = FEAT_INFLUENCE_SAFIYA_DEVOTED;
	else if (sTag == TAG_GANN)
		iFeat = FEAT_INFLUENCE_GANN_DEVOTED;
	else if (sTag == TAG_OKKU)
		iFeat = FEAT_INFLUENCE_OKKU_DEVOTED;
	else if (sTag == TAG_DOVE)
		iFeat = FEAT_INFLUENCE_DOVE_DEVOTED;
	else if (sTag == TAG_ONEOFMANY)
		iFeat = FEAT_INFLUENCE_ONEOFMANY_DEVOTED;
	return (iFeat);		
}


int GetRomanceFeat(string sTag)
{
	int iFeat = -1;
	if (sTag == TAG_SAFIYA)
		iFeat = FEAT_INFLUENCE_SAFIYA_ROMANCE;
	else if (sTag == TAG_GANN)
		iFeat = FEAT_INFLUENCE_GANN_ROMANCE;
	return (iFeat);		
}

const string VAR_SAVED_REBELLIOUS_LEVEL_PREFIX	= "SAVED_REBELLIOUS_LEVEL";

int GetSavedRebelliousLevelByTag(string sCompanionTag)
{
	string sVarName = VAR_SAVED_REBELLIOUS_LEVEL_PREFIX + sCompanionTag;
	int iRet = GetGlobalInt( sVarName );
	return (iRet );	
}

void SetSavedRebelliousLevelByTag(string sCompanionTag, int iValue)
{
	string sVarName = VAR_SAVED_REBELLIOUS_LEVEL_PREFIX + sCompanionTag;
	SetGlobalInt( sVarName,  iValue);
}


void InitiateLeaveParty(object oTarget)
{
	//AssignCommand (oTarget, SpeakString ("That's the last straw!  I am so out of here!!!!"));
	
	string sTargetTag 	= GetTag(oTarget);
	string sLeavePartyDialogName = "gl_" + sTargetTag + "_leave";
	//PrettyDebug("sLeavePartyDialogName = " + sLeavePartyDialogName);
	CreateIPSpeaker( sTargetTag, sLeavePartyDialogName, GetLocation( OBJECT_SELF ), 1.75f );
}

void SetPlayerInventoryAccess(object oTarget, int bCanAccessInventory)
{
	int bDisableEquip = !bCanAccessInventory;
	// TRUE = equipment disabled
	SetLocalInt(oTarget, "X2_JUST_A_DISABLEEQUIP", bDisableEquip);
}

void SetIsPossessable(object oTarget, int bPossessable)
{
	int bBlocked = !bPossessable;
	SetIsCompanionPossessionBlocked(oTarget, bBlocked);
}

void voidFeatAdd(object oCreature, int iFeatId, int bCheckRequirements)
{
		int bSuccess = FeatAdd(oCreature, iFeatId, bCheckRequirements);
		PrettyDebug("FeatAdd() bSuccess = " + IntToString(bSuccess));
}

// applies the apropriate effects from influence to the companions
// and removes any effects that no longer apply.
void DoInfluenceEffects(object oTarget = OBJECT_SELF)
{
	if (!GetIsObjectValid(oTarget))
	{
		PrettyDebug("Target of Influence effects not found.");
		return;
	}
    //SpawnScriptDebugger();
	string sTargetTag 	= GetTag(oTarget);
	int iInfluence 		= GetInfluenceByTag(sTargetTag);
	int iInfluenceBand 	= GetInfluenceBand(iInfluence);
	
	int bPossessable 		= TRUE;
	int bCanAccessInventory = TRUE;
	int bWillLeaveParty 	= FALSE;
	int iNewFeat 			= -1;
				
	
	// special handling for rebellion
	// if we have left rebellious then saved rebellion level is cleared.
	if (iInfluenceBand != INFLUENCE_REBELLIOUS)
		SetSavedRebelliousLevelByTag(sTargetTag, 0);
		 
	// add current influence effects
	switch (iInfluenceBand)
	{
		case INFLUENCE_REBELLIOUS:
		{
			// unpossesable
			bPossessable = FALSE;
			
			// can't access inventory
			bCanAccessInventory = FALSE;
			
			// if saved rebellious level = 0, save rebellious level
			// otherwise, if saved rebellious level < current rebellious level, then initiate leave party dialog
			int iSavedRebelliousLevel = GetSavedRebelliousLevelByTag(sTargetTag);
			if (iSavedRebelliousLevel == 0)
				SetSavedRebelliousLevelByTag(sTargetTag, iInfluence);
			else
			{	// check if we were previously rebellious and it has gone down more.
				// rebellious starts at -76
				if ((iInfluence<=-100) || (iInfluence < iSavedRebelliousLevel))
				{
					bWillLeaveParty = TRUE;
				}
			}
		}			
			break;
			
		case INFLUENCE_DISILLUSIONED:
			// unpossessable
			bCanAccessInventory = FALSE;
			break;
			
		case INFLUENCE_WAVERING:
			// no effect
			break;
			
		case INFLUENCE_NEUTRAL:
			// no effect
			break;
			
		case INFLUENCE_SUPPORTIVE:
			// add supportive feat
			iNewFeat = GetSupportiveFeat(sTargetTag);
			break;
			
		case INFLUENCE_LOYAL:
			// add loyal feat
			iNewFeat = GetLoyalFeat(sTargetTag);
			break;
			
		case INFLUENCE_DEVOTED:
			// add devoted feat
			iNewFeat = GetDevotedFeat(sTargetTag);
			break;
	}
	
	SetPlayerInventoryAccess(oTarget, bCanAccessInventory);
	SetIsPossessable(oTarget, bPossessable);
	if (bWillLeaveParty)
		InitiateLeaveParty(oTarget);

	// remove all previous influence feats
	RemoveInfluenceFeats(oTarget);
	// add new feat if any
	if (iNewFeat > -1)
	{
		int bSuccess = FeatAdd(oTarget, iNewFeat, FALSE);
		PrettyDebug("FeatAdd() bSuccess = " + IntToString(bSuccess));
		//AssignCommand(oTarget, voidFeatAdd(oTarget, iNewFeat, FALSE));
		if (iNewFeat == FEAT_INFLUENCE_SAFIYA_DEVOTED || iNewFeat == FEAT_INFLUENCE_SAFIYA_LOYAL)
		{
			bSuccess = FeatAdd(oTarget, FEAT_IMPROVED_EMPOWER_SPELL, FALSE);
		}
		PrettyDebug("FeatAdd() bSuccess = " + IntToString(bSuccess));
	}
    // *** GUI                
    // Set the variable indicating this char should display influence on character screen
    SetLocalInt(oTarget, "GUI_DisplayInfluence", TRUE);
    
	// set up influence for GUI
	SetLocalInt(oTarget, "influence", iInfluence );
    
	// set up influence band str ref for GUI
    int iInfluenceBandStrRef = GetInfluenceBandStrRef(iInfluenceBand);
	SetLocalInt(oTarget, "influence_band", iInfluenceBandStrRef );
}


// Set PC's influence of companion to nAmount ( capped )
void SetInfluenceByCode( string sCompanionCode, int nAmount )
{
	// sCompanionCode = GetFirstChars(sCompanionCode);
    string sCompanionTag = GetCompanionTagByCode(sCompanionCode);
	string sVarName = GetInfluenceVarNameByTag( sCompanionTag );
	
	int	iInfluenceMin = INFLUENCE_MIN;
	
	// Safiya never goes below disillusioned.
	if (sCompanionTag == TAG_SAFIYA)
		iInfluenceMin = INFLUENCE_RANGE_REBELLIOUS + 1;
		
	if ( nAmount < iInfluenceMin )
		nAmount = iInfluenceMin;
	else if ( nAmount > INFLUENCE_MAX )
		nAmount = INFLUENCE_MAX;

	//int iOldBand = GetInfluenceBandByCode(sCompanionCode);
	//int iNewBand = GetInfluenceBand(nAmount);

	// update influence var
	SetGlobalInt( sVarName, nAmount );
	
	object oCompanion = GetObjectByTag(sCompanionTag);
	if (GetIsObjectValid(oCompanion))
    {
        
    	// check if we have changed influence bands
    	// (no need to run this if we haven't unless its rebellious)
        // Change: Do this all the time as we want to update the vars on the companion
        // for use by the GUI.
    	//if ((iOldBand != iNewBand) || (iNewBand == INFLUENCE_REBELLIOUS ))
    	//{
			AssignCommand(oCompanion, DoInfluenceEffects());
	    //}
    }        
}

// Increase PC's influence of companion by nInfluence ( capped )
void ModifyInfluenceByCode( string sCompanionCode, int nDelta )
{
	int nInfluence = GetInfluenceByCode(sCompanionCode) + nDelta;
	SetInfluenceByCode(sCompanionCode, nInfluence);

	// reget the influence - may be different than what we just calculated because of various mins and maxs
	int nInfluenceTotal = GetInfluenceByCode(sCompanionCode);
	ReportInfluenceChange(sCompanionCode, nDelta, nInfluenceTotal);
}


// This is a number between -5 and 5 thar represents the level of magnitude of the change.
// Returns the amount of influence represented.
int MagnitudeConversion(int iMagnitude)
{
	if (iMagnitude == 0)
		return (0);	// make sure we don't divide by 0.
		
	int iSign = iMagnitude/abs(iMagnitude);	// 1 or -1
	int iAmount = 0;

	
	switch (abs(iMagnitude))
	{
		case 1:	iAmount = INFLUENCE_MAGNITUDE_1; 	break;
		case 2:	iAmount = INFLUENCE_MAGNITUDE_2; 	break;
		case 3:	iAmount = INFLUENCE_MAGNITUDE_3; 	break;
		case 4:	iAmount = INFLUENCE_MAGNITUDE_4; 	break;
		// Values outside the range default to +/- 5 magnitude
		default:	iAmount = INFLUENCE_MAGNITUDE_5; 	break;
 	}
	
	iAmount = iAmount * iSign; // reapply sign
	
	return (iAmount);
}

int GetInfluenceBandByCode(string sCompanionCode)
{
	int iInfluence = GetInfluenceByCode(sCompanionCode);
	int iInfluenceBand = GetInfluenceBand(iInfluence);
	return (iInfluenceBand);
}

// Returns the corresponding band for this level of influence
int GetInfluenceBand(int iInfluence)
{
	int iInfluenceBand;
	
	if (iInfluence <= INFLUENCE_RANGE_REBELLIOUS)
		iInfluenceBand = INFLUENCE_REBELLIOUS;
		
	else if (iInfluence <= INFLUENCE_RANGE_DISILLUSIONED)
		iInfluenceBand = INFLUENCE_DISILLUSIONED;

	else if (iInfluence <= INFLUENCE_RANGE_WAVERING)
		iInfluenceBand = INFLUENCE_WAVERING;
				
	else if (iInfluence <= INFLUENCE_RANGE_NEUTRAL)
		iInfluenceBand = INFLUENCE_NEUTRAL;

	else if (iInfluence <= INFLUENCE_RANGE_SUPPORTIVE)
		iInfluenceBand = INFLUENCE_SUPPORTIVE;

	else if (iInfluence <= INFLUENCE_RANGE_LOYAL)
		iInfluenceBand = INFLUENCE_LOYAL;
		
	else 
		iInfluenceBand = INFLUENCE_DEVOTED;
				
	return (iInfluenceBand);
}

int GetInfluenceBandStrRef(int iInfluenceBand)
{
    int iStrRef=0;
	switch (iInfluenceBand)
	{
		case INFLUENCE_REBELLIOUS:      iStrRef = STR_REF_INFLUENCE_REBELLIOUS;		break;
		case INFLUENCE_DISILLUSIONED:   iStrRef = STR_REF_INFLUENCE_DISILLUSIONED;	break;
		case INFLUENCE_WAVERING:	    iStrRef = STR_REF_INFLUENCE_WAVERING;   	break;
		case INFLUENCE_NEUTRAL:		    iStrRef = STR_REF_INFLUENCE_NEUTRAL;		break;
		case INFLUENCE_SUPPORTIVE:	    iStrRef = STR_REF_INFLUENCE_SUPPORTIVE;		break;
		case INFLUENCE_LOYAL:		    iStrRef = STR_REF_INFLUENCE_LOYAL;			break;
		case INFLUENCE_DEVOTED:		    iStrRef = STR_REF_INFLUENCE_DEVOTED;		break;
	}
    return (iStrRef);
}

// Gets initial influence for new companions based on PC's Charisma and relative alignment
int GetInfluenceAdjustment(object oPC, object oNPC)
{

	// Gets CH bonus based on raw Charisma, adjusted by race but not items, magic or other effects.
	int nAdjust = GetAbilityScore(oPC, ABILITY_CHARISMA, TRUE);
	int nRace = GetSubRace(oPC);
	if (nRace == RACIAL_SUBTYPE_GRAY_DWARF || nRace == RACIAL_SUBTYPE_SVIRFNEBLIN)
		nAdjust -= 4;
	else if (nRace == RACIAL_SUBTYPE_SHIELD_DWARF || nRace == RACIAL_SUBTYPE_HALFORC || nRace == RACIAL_SUBTYPE_GRAYORC || nRace == RACIAL_SUBTYPE_TIEFLING || nRace == RACIAL_SUBTYPE_AIR_GENASI || nRace == RACIAL_SUBTYPE_EARTH_GENASI || nRace == RACIAL_SUBTYPE_FIRE_GENASI || nRace == RACIAL_SUBTYPE_WATER_GENASI)
		nAdjust -= 2;
	else if (nRace == RACIAL_SUBTYPE_DROW || nRace == RACIAL_SUBTYPE_AASIMAR || nRace == RACIAL_SUBTYPE_YUANTI)
		nAdjust += 2;
	nAdjust = nAdjust / 2 - 5;

	// Alignment influence adjustment for each axis - non-neutral alignments only
	if (GetAlignmentGoodEvil(oPC) != ALIGNMENT_NEUTRAL)
	{
		if (GetAlignmentGoodEvil(oPC) == GetAlignmentGoodEvil(oNPC))
		{
			nAdjust += 2;
		}
		else if (GetAlignmentGoodEvil(oNPC) != ALIGNMENT_NEUTRAL && GetAlignmentGoodEvil(oPC) != GetAlignmentGoodEvil(oNPC))
		{
			nAdjust -= 2;
		}
	}
	if (GetAlignmentLawChaos(oPC) != ALIGNMENT_NEUTRAL)
	{
		if (GetAlignmentLawChaos(oPC) == GetAlignmentLawChaos(oNPC))
		{
			nAdjust += 2;
		}
		else if (GetAlignmentLawChaos(oNPC) != ALIGNMENT_NEUTRAL && GetAlignmentLawChaos(oPC) != GetAlignmentLawChaos(oNPC))
		{
			nAdjust -= 2;
		}
	}
	return (nAdjust);
}